/*
	File:		CADocPriv.h

	Contains:	Common code for CALib (private header file, includes CALib.h

	Written by:	Rick Badertscher

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):


		 <1>	11/16/95	RB		Modifications to clipboard and dragdrop to support
		 							CAGetClipboardDocument() and CAGetDragDropDocument() 
		 							API changes.
		 <0>	7/20/95		RB		Written
	To Do:
		- Compile
		- Implement
*/

#ifndef _CADOCPRIV_
#define _CADOCPRIV_

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif


#ifndef _USERSRCM_
#include <UseRsrcM.h>
#endif 

#ifndef _PLFMFILE_
#include <PlfmFile.h>
#endif 

//-------------------------------------------------------------------------


class ODDraft;
class ODContainer;
class ODFrame;
class ODStorageUnit;
class ODWindow;
class ODPart;
class OrderedCollection;
class PlatformFile;
class ODDragAndDrop;
class ODClipboard;




//=========================================================================
// Utility Routine Prototypes
//=========================================================================

void			CloneDraftToFileContainer(			ODDraft*		srcDraft, PlatformFile* file);
void			CloneDraftToMemoryContainer(		ODDraft*		srcDraft, ODHandle* handle);
PlatformFile*	FileContainerFromMemoryContainer(	ODHandle		memoryCtrStream);
ODHandle		MemoryContainerFromFileContainer(	PlatformFile* 	fileCtr);


//=========================================================================
// Constants
//=========================================================================

enum CADocumentType
{
	kCAProxyDocument,
	kCAPartDocument,
	kCAClipboardDocument,
	kCADragDropDocument
};

enum CAContainerType
{
	kCATempMemoryContainer,
	kCAInPlaceMemoryContainer,
	kCATempFileContainer,
	kCAInPlaceFileContainer
};

enum CADragState
{
	kDragLoading,
	kDragUnloading,
	kDragDragging,
	kDragInternalDropDone,
	kDragIdle
};


//=========================================================================

#define ASSERT_TYPE(doc,type)													\
																				\
	if ( ((CADocument*)doc->GetDocumentType != type) &&							\
		 (! ((type == kCAAppDocument) &&										\
		    ((CADocument*)doc->GetDocumentType() == kCAODCtrDocument))))		\
				DebugStr ("\p Invalid document type for operation!");


#define kODCloneAll 8UL

//=========================================================================
// class CADocument (CADocumentRef == CADocument*)
//=========================================================================


class CADocument
{

	public:
	
							CADocument (		void);
							~CADocument();
						
	// for initing CADocument's with file based temp container
			void				InitCADocument(		ODDraft*			draft,
													const ODFileSpec*	fileSpec,
													PlatformFile*		tempFile,
													Boolean				deleteFile,
													CADocumentType		docType,
													CAContainerType		ctrType);
												
	// for initing CADocument's with file memory temp container
			void				InitCADocument(		ODDraft*			draft,
													const ODFileSpec*	fileSpec,
													ODHandle			tempHandle,
													CADocumentType		docType,
													CAContainerType		ctrType);

	virtual Boolean				Close(				void);

	// Content value methods
			void				FocusContentStream(	void);
			void				SetCurrentKind(		ODISOStr	kind);
	inline	CAISOStr			GetCurrentKind(		void)				{ return (fCurrentKind);}

	// Saving
	virtual	CASize				StreamToFile(		PlatformFile*	file);
	virtual	CASize				StreamToMemory(		Handle*			containerHandle, CASize offset);

	virtual CASize				SaveAsFile(			PlatformFile*	file);
	virtual CASize				Save(				void);

			void				AddLimboFrame(		ODFrame*		frame);
	inline	OrderedCollection*	GetLimboFrames(	void)					{ return (fLimboFrames);}
			
	// Getters & Setters
	inline	void				SetCAFileSpec(		const ODFileSpec* fileSpec)	{ fCAFileSpec = *fileSpec;}
	inline	ODFileSpec			GetCAFileSpec(		void)				{ return (fCAFileSpec);}
	
	inline	PlatformFile*		GetFile(			void)				{ return (fTmpCtrFile);}
	
	inline	void				SetStreamOffset(	long offset)		{ fStreamOffset = offset;}
	inline	ODULong				GetStreamOffset(	void)				{ return fStreamOffset;}
	
	inline	ODWindow*			GetWindow(			void)				{ return fWindow;}
	inline	void				SetWindow(			ODWindow* window)	{ fWindow = window;}
	
	inline	CADocumentType		GetDocumentType(	void)					{ return fDocType;}
	inline	void				SetDocumentType(	CADocumentType docType)	{ fDocType = docType;}
	
	inline	CAContainerType		GetContainerType(	void)					{ return fCtrType;}
	inline	void				SetContainerType(	CAContainerType ctrType){ fCtrType = ctrType;}
	
	inline	void				SetDeleteFile(		Boolean deleteFile)		{ fDeleteFile = deleteFile;}
	virtual CACloneKind			GetCloneKind(		void);
	virtual ODStorageUnit*		GetRootPartSU(		void);
			
			ODDraft*			GetDraft(			void);
			
			ODPart*				AcquireRootPart(	void);
			ODFrame*			AcquireDisplayFrame(void);


	protected:
	
		ODStorageUnit*			fRootPartSU;
		CACloneKind				fCloneKind;
		ODULong					fStreamOffset;
		
	private:
	
		PlatformFile*			fTmpCtrFile;
		ODHandle				fTmpCtrHandle;
		ODFileSpec				fCAFileSpec;
		ODWindow*				fWindow;
		Boolean					fDeleteFile;
		ODISOStr				fCurrentKind;
		CADocumentType			fDocType;
		CAContainerType			fCtrType;
		OrderedCollection*		fLimboFrames;

};



//=========================================================================
// class CAClipboardDocument
//=========================================================================

class CAClipboardDocument : public CADocument
{

	public:
							CAClipboardDocument (	ODClipboard* clipboard);
							~CAClipboardDocument();
							
			void			PrepareForClone(		CACloneKind		cloneKind);
			Boolean			Close(					void);
	inline	CACloneKind		GetCloneKind(			void)		{ return fCloneKind;}
		
	virtual CADocumentType	GetDocumentType(		void);
	virtual	CASize			StreamToFile(			PlatformFile*	file);

		
	private:
	
		CACloneKind				fCloneKind;
		
};

//=========================================================================
// class CADragDropDocument
//=========================================================================
class CADragDropDocument : public CADocument
{

	public:

							CADragDropDocument(	ODDragAndDrop* dragDrop);
							~CADragDropDocument();
	
			void			Acquire(			ItemReference	theItem);
			void			PrepareForClone(	CACloneKind		cloneKind);

			Boolean			Close(				void);
	
	inline	CACloneKind		GetCloneKind(		void)						{ return fCloneKind;}

			void			AddDraggingFrame(	ODFrame* frame);
			void			SetDragging(		void);
	
	virtual CADocumentType	GetDocumentType(	void);


	private:
	
		CACloneKind				fCloneKind;
		OrderedCollection*		fDraggingFrames;
		short					fRefCount;
		ItemReference			fDragItem;
		Boolean					fInitiated;
		
};



#endif
